package com.loafle.overflow.rpc.api;

import com.fasterxml.jackson.core.JsonEncoding;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.util.JSONPObject;
import com.googlecode.jsonrpc4j.spring.AutoJsonRpcServiceImpl;
import com.loafle.overflow.rpc.crawler.Crawler;
import com.loafle.overflow.rpc.model.Input;
import com.loafle.overflow.rpc.model.Output;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Service
@AutoJsonRpcServiceImpl
public class RemoteImpl implements Remote {

    private Map<String, Crawler> crawlerMap = null;


    @Override
    public Output init(Input input) throws Exception {

        for (String v : input.getPaths()) {
            System.out.println(v);
        }
        Output out = new Output();
        out.setStartDate(new Date());
        out.setEndDate(new Date());
        out.setData("Method init".getBytes());
        return out;
    }

    @Override
    public Output add(Input input) throws Exception {
        Crawler crawler = this.crawlerMap.get(input.getName());
        if(crawler == null) {
            System.out.println(input.getName() + "_crawler == null");
            return null;
        } else {
            Output out = new Output();
            out.setStartDate(new Date());

            Object obj = crawler.add(input.getId());

            out.setEndDate(new Date());
            out.setData(convertObjectToJsonBytes(obj));

            return out;
        }
    }

    @Override
    public int addDelegate(String name, Crawler crawler) {

        if(this.crawlerMap == null) {
            this.crawlerMap = new HashMap<String, Crawler>();
        }

        if(this.crawlerMap.containsKey(name)) {
            return 1;
        }

        this.crawlerMap.put(name, crawler);

        return 0;
    }

    @Override
    public Output get(Input input) throws Exception {

        Crawler crawler = this.crawlerMap.get(input.getName());
        if(crawler == null) {
            System.out.println(input.getName() + "_crawler == null");
            return null;
        } else {

            Output out = new Output();
            out.setStartDate(new Date());

            Object result = crawler.get(input.getId());
            out.setEndDate(new Date());

            out.setData(convertObjectToJsonBytes(result));
            return out;
        }

    }

    @Override
    public Output remove(Input input) throws Exception {
        Crawler crawler = this.crawlerMap.get(input.getName());
        if(crawler == null) {
            System.out.println(input.getName() + "_crawler == null");
            return null;
        } else {
            Output out = new Output();
            out.setStartDate(new Date());

            Object obj = crawler.remove(input.getId());
            out.setEndDate(new Date());
            out.setData(convertObjectToJsonBytes(obj));
            return out;
        }
    }

    private byte[] convertObjectToJsonBytes(Object object) throws Exception {

        if(object == null) {
            return null;
        }

        ObjectMapper mapper = new ObjectMapper();

        String jsonInString = mapper.writeValueAsString(object);

        return jsonInString.getBytes();
    }
}